var RE = {};

// 当前选中区域
RE.currentSelection = {
    "startContainer": 0,
    "startOffset": 0,
    "endContainer": 0,
    "endOffset": 0
};

RE.editor = document.getElementById('editor');

RE.focus = function(){
    RE.editor.focus()
}

// 设置字体颜色
RE.setBaseTextColor = function(color) {
    RE.editor.style.color  = color;
}

// 设置行间的距离(行高)
RE.setBaseLineHeight = function(height) {
    RE.editor.style.lineHeight  = height;
}

// 设置字体大小
RE.setBaseFontSize = function(size) {
    RE.editor.style.fontSize = size;
}

// 设置内边距
RE.setPadding = function(left, top, right, bottom) {
  RE.editor.style.paddingLeft = left;
  RE.editor.style.paddingTop = top;
  RE.editor.style.paddingRight = right;
  RE.editor.style.paddingBottom = bottom;
}

// 设置背景色
RE.setBackgroundColor = function(color) {
    RE.editor.style.backgroundColor = color;
}

// 设置编辑器宽度
RE.setWidth = function(size) {
    RE.editor.style.minWidth = size;
}

// 设置编辑器高度
RE.setHeight = function(size) {
    RE.editor.style.height = size;
}

// 设置编辑器文本对齐方式
RE.setTextAlign = function(align) {
    RE.editor.style.textAlign = align;
}

// 设置编辑器垂直对齐方式
RE.setVerticalAlign = function(align) {
    RE.editor.style.verticalAlign = align;
}

// 设置默认文本
RE.setPlaceholder = function(placeholder) {
    RE.editor.setAttribute("placeholder", placeholder);
}

// 设置编辑器内容是否可编辑
RE.setInputEnabled = function(inputEnabled) {
    RE.editor.contentEditable = String(inputEnabled);
}

// 将编辑器的内容传递给客户端
RE.callback = function () {
    opEditor('callback_edit_html', RE.getArticleInfo());
}

// 设置html内容
RE.setHtml = function (contents) {
    RE.editor.innerHTML = decodeURIComponent(contents.replace(/\+/g, '%20'));
    // 触发回调内容变化及图片等信息变化
    RE.callback();
}

// 获取html内容
RE.getHtml = function () {
    return RE.editor.innerHTML;
}

// 获取纯文本内容
RE.getText = function () {
    return RE.editor.innerText;
}

// 撤销
RE.undo = function () {
    document.execCommand('undo', false, null);
}

// 恢复
RE.redo = function () {
    document.execCommand('redo', false, null);
}

// 加粗
RE.setBold = function () {
    document.execCommand('bold', false, null);
    RE.enabledEditingItems();
}

// 插入html内容
RE.insertHTML = function (html) {
    document.execCommand('insertHTML', false, html);
}

// 插入链接（链接不可中间插入内容）
RE.insertLink = function (url, title) {
    // 创建一个a标签
    var el = document.createElement("a");
    el.innerText = `${title || url}`;
    el.setAttribute("contenteditable", "false");
    el.setAttribute("href", url);
    el.setAttribute("title", title);
    el.setAttribute("data-type", 'link');
    el.setAttribute("data-id", '');
    el.setAttribute("data-value", url);

    insertHTMLNode(el);
}

// 插入分割线
RE.insertHorizontalRule = function (url) {
    let p = document.createElement("p");
    // 创建一个img标签
    var img = document.createElement("img");
    img.setAttribute("src", url);
    img.setAttribute("alt", '分割线');
    img.setAttribute("data-type", 'divider');
    img.setAttribute("data-id", '');
    img.setAttribute("data-value", url);
    img.setAttribute("style", "width:100%;height:auto;margin:10px auto;");
    p.appendChild(img);

    insertHTMLNode(p);
    RE.insertHTML('<br>'); // 回车换行
}

// rbg转16进制颜色
function rgbToHex(str){
    let reg = /^(rgb|RGB)/;
    if(!reg.test(str)){return;}
    var arr = str.slice(4, str.length-1).split(",")
    let color = '#';
    arr.map(d => color += Number(d).toString(16).padStart(2, 0));
    return color;
}

// 获取光标字体颜色
function getSelectedFontColor(){
    var node,selection;
    if (window.getSelection) {
        selection = getSelection();
        node = selection.anchorNode;
    }
    if (!node && document.selection) {
        selection = document.selection
        var range = selection.getRangeAt ? selection.getRangeAt(0) : selection.createRange();
        node = range.commonAncestorContainer ? range.commonAncestorContainer :
        range.parentElement ? range.parentElement() : range.item(0);
    }
    let color = null;
    if (node) {
        var item =  (node.nodeName == "#text" ? node.parentNode : node);
        color = item.getAttribute("color") || item.style["color"];
        if(color.indexOf('rgb') > -1){
            color = setRgbTo16(color);
        }
    }
    return color
}

// 已启用编辑项目
RE.enabledEditingItems = function (e) {
    var items = [];

    if (document.queryCommandState('bold')) {
        items.push('bold'); // 加粗
    }

    let color = document.queryCommandValue('foreColor'); // 获取当前选中区的前景(文本)颜色
    if (color) {
        color = rgbToHex(color);
        items.push(color);
        //RE.setTextColor(color);
    } else {
        //RE.setTextColor("#333333");
    }

    opEditor('callback_selection_style', {items: items});
}

// 设置文本颜色
RE.setTextColor = function(color) {
    document.execCommand('foreColor', false, color);
}

// 插入@
RE.insertUser = function(id = 0, username = '') {
    // 创建一个@标签
    let span = document.createElement("span");
    span.classList.add("user-link");
    span.innerText = `@${username}`;
    span.setAttribute("contenteditable", "false"); // 点击时需要整体选中，删除时需要整体删除。
    span.setAttribute("data-type", '@');
    span.setAttribute("data-id", id);
    span.setAttribute("data-value", username);

    var selection = window.getSelection();
    var range = selection.getRangeAt(0);
    const textNode = range.startContainer;
    let text = textNode.nodeValue;
    // 判断为字符节点，且最后一个字符为@，则删除@符号
    if(textNode.nodeType == 3 && text.slice(-1) == '@'){
        // 如果是键盘触发的默认删除面前的@
        range.setStart(textNode, range.endOffset - 1);
        range.setEnd(textNode, range.endOffset);

        // 删除原有内容
        range.deleteContents();
    }
    insertHTMLNode(span)
}

// 插入图片
RE.insertImage = function(url, title = '') {
    let p = document.createElement("p");
    // 创建一个img标签
    var img = document.createElement("img");
    img.setAttribute("src", url);
    if(title)
        img.setAttribute("alt", title);
    img.setAttribute("data-type", 'img');
    img.setAttribute("data-id", '');
    img.setAttribute("data-value", url);
    img.setAttribute("style", "width:25%;height:auto;margin:10px 0;border-radius: 8px;");
    p.appendChild(img);

    insertHTMLNode(p);
    RE.insertHTML('<br>'); // 回车换行
}

// 插入表情包
RE.insertEmote = function(url, key, height = 55) {
    // 创建一个img标签
    var img = document.createElement("img");
    img.setAttribute("src", url);
    img.setAttribute("alt", key);
    img.setAttribute("class", 'emote');
    img.setAttribute("data-type", 'emote');
    img.setAttribute("data-id", key);
    img.setAttribute("data-value", url);
    if(height){
        img.setAttribute("height", height); // 狗头32，其他55
    }

    insertHTMLNode(img);
}

// 获取帖子内容 - 富文本内容、图片信息
RE.getArticleInfo = function() {
    let imginfo = []
    const imgs = RE.editor.querySelectorAll('img'); // 获取富文本编辑器内所有img标签
    var img = null
    for(var i = 0; i < imgs.length; i++) {
        img = imgs[i];
        // 如果是表情包或者分割线，则过滤掉
        if(img.getAttribute('data-type') != 'divider' && img.getAttribute('data-type') != 'emote'){
            let params = {
                url: img.src.replace('file://',''),
                width: img.naturalWidth,    // 图片原始宽度
                height: img.naturalHeight,  // 图片原始高度
            }
            imginfo.push(params)
        }
    }
    const users = RE.editor.querySelectorAll('.user-link'); // 获取富文本编辑器内所有@标签
    var userList = {}
    for(var i = 0; i < users.length; i++) {
        let user = users[i];
        // 如果是@用户，则记录用户的id和name
        if(user.getAttribute('data-type') == '@') {
            if(!userList[user.getAttribute('data-value')]) { // 已存在的用户名称，避免重复添加
               userList[user.getAttribute('data-value')] = +user.getAttribute('data-id')
            }
        }
    }
    return {
        txt: RE.getText(), // 富文本纯文本内容
        html: encodeURIComponent(RE.getHtml()), // 富文本内容
        imginfo: imginfo, // 帖子图片信息（url、width、height）
        atUsers: userList // @用户列表
    }
}

// 添加监听事件
RE.editor.addEventListener("input", function (e) {
    // 输入@符号 唤起@用户列表
    if(e.data == '@'){
        // 唤起用户列表 - 客户端需要回调RE.insertUserName方法
        opEditor('callback_open_friend_list');
    }
    RE.callback();
    RE.enabledEditingItems(e);
});
RE.editor.addEventListener("keyup", function (e) {
    var KEY_LEFT = 37,
        KEY_RIGHT = 39;
    if (e.which == KEY_LEFT || e.which == KEY_RIGHT) {
        RE.enabledEditingItems(e);
    }
});
RE.editor.addEventListener("click", function (e) {
    // 需要检查点击的元素节点，判断是否调用客户端方法（如点击链接、@用户等）
    if(e.target && e.target.dataset){ // 判断是否有自定义属性
        const { type, id, value } = e.target.dataset;
        const title = e.target.title || '';
        switch (type) {
            case 'img':
                // TODO 调用客户端方法，查看图片
                break;
            case 'link':
                e.preventDefault();// 阻止浏览器默认事件（超链接不可点击跳转）
                break;
            case 'divider':
                break;
            case '@':
                break;
            case 'emote':
                break;
            default:
                break;
        }
    }
    RE.enabledEditingItems();
});
RE.editor.addEventListener("paste", function (e) {
    onPaste(e)
});

//格式化粘贴文本方法
function onPaste(event) {
    var e = event || window.event
    // 阻止默认粘贴
    e.preventDefault();
    // 粘贴事件有一个clipboardData的属性，提供了对剪贴板的访问
    // clipboardData的getData(format) 从剪贴板获取指定格式的数据
    var text = (e.originalEvent || e).clipboardData.getData('text/plain');
    //清除回车
    text = text.replace(/\[\d+\]|\n|\r/ig, "")
    // 插入
    document.execCommand("insertText", false, text);
}

// 插入html节点
function insertHTMLNode(node) {
    // 如果有选中的内容，就替换掉
    var selection = window.getSelection();
    var range = selection.getRangeAt(0);

    // 删除原有内容
    range.deleteContents();

    range.insertNode(node);

    //  移动光标到下一个输入点
    selection.collapseToEnd();

    node.scrollIntoView();

    RE.callback();
}

RE.prepareInsert = function() {}

/**
 * 编辑器交互协议
 * @param {*} type 交互协议类型
 * @param {*} params 交互协议参数
 * callback_edit_html: 编辑器富文本内容 params: {html: ''}
 * callback_selection_style: 编辑器启用项目 params: {items: ''}
 * callback_open_link_editor: 打开链接编辑弹框 params: {url: '', title: ''}
 * callback_open_friend_list: 打开好友列表 params: {id: '', name: ''}
 */
function opEditor(type, params) {
    setTimeout(() => {
        window.location.href = "op-editor://" + encodeURIComponent(JSON.stringify({
            "protocol_type": type,
            ...params
        }))
    }, 50)
}